/*
 * HT3010 initialize
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#include "set3010.h"

static struct file_operations set3010_fops = {
  owner: THIS_MODULE
};

void release3010(void)
{
  release_region( ISAIO8_BASE  + P3010_1, 2 );
}

void free_all(void)
{
  unregister_chrdev (SET3010_MAJOR, CHRDEV);
}


unsigned char config_table_1[]={
	0x03,0xf4,		/* CR03: 11110100 */
	0x07,0x80,		/* CR07: 10000000 disable auto power OFF */
	0x20,0x00,		/* CR20: 00000000 disable FDC */
	0x23,0x00,		/* CR23: 00000000 disable PRT */
	0x24,COMADR3F8,	/* CR24: COM1 Address A[9:3] */
	0x25,COMADR2F8,	/* CR25: COM2 Address A[9:3] */
	0x26,0x00,		/* CR26: 00000000 no DMA for FDC/PRT */
	0x27,0x00,		/* CR27: 00000000 no IRQ for FDC/PRT */
	0x28,IRQ4_IRQ3,	/* CR28: IRQ channels COM1 and COM2 */
	0xaa
};

void cfg3010(int cfgport3010,unsigned char *config_table)
{
	int i=0;
	unsigned char data;
	/* enter configuration mode */
	outp(cfgport3010,0x55);
	outp(cfgport3010,0x55);
	while (data=config_table[i], outp(cfgport3010,data), data != 0xaa) {
		i++;
		outp(cfgport3010+1,config_table[i]);
		i++;
	}
}

int __init set3010_init (void)
{

  if (register_chrdev (SET3010_MAJOR, CHRDEV, &set3010_fops)) {
    printk (KERN_WARNING CHRDEV ": Unable to get major %d.\n",SET3010_MAJOR);
    return -EIO;
  }

  if (!request_region( ISAIO8_BASE  + P3010_1 , 2 ,"HT3010CFG" )){
    printk (KERN_WARNING CHRDEV ": ISAIO8_0x%x is not free.\n",P3010_1);
    release3010();
    return -EIO;
  }
  cfg3010(P3010_1,config_table_1);

  printk (KERN_WARNING CHRDEV ": HT3010 was configured for 2 serial ports.\n");

  printk (KERN_INFO VERSION "\n");
  release3010();
  return 0;
}

void __exit set3010_cleanup (void)
{
  free_all();
}

module_init (set3010_init);
module_exit (set3010_cleanup);
